
/*!
 * async-util
 * JavaScript async utilities library
 *
 * Copyright (c) 2011 Enrico Marino
 * MIT license
 */

!function (exports) {

  exports.async = {};

  /**
   * Library version
   */
  async.version = 0.0.1;

  /**
   * Namespaces
   */
  async.parallel = {};
  async.series = {};

  /**
   * Apply 'iterator' to each item in 'array' in parallel
   * and call 'callback' when done
   *
   * @param {Array} array array
   * @param {Function} iterator iterator
   * @param {Function} callback callback
   */

  async.each =
  async.parallel.each = function (array, iterator, callback) {
    var completed = 0
      , len = array.length
      , i
      ;

    function complete (err) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      completed += 1;
      if (completed === len) {
        callback();
        return;
      }
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterator(array[i], complete);
      }
    }
  };

  /**
   * Apply 'iterator' to each item in 'array' in series
   * and call 'callback' when done
   *
   * @param {Array} array array
   * @param {Function} iterator iterator
   * @param {Function} callback callback
   */

  async.series.each = function (array, iterator, callback) {

    var completed = 0
      , len = array.length
      ;

    function iterate () {

      function complete (err) {
        if (err) {
          callback(err);
          callback = function () {};
          return;
        }
        completed += 1;
        if (completed === len) {
          callback();
          return;
        }
        iterate();
      }

      iterator(array[completed], complete);
    }

    iterate();
  };

  /**
   * Apply 'iterator' to the first 'limit' items in 'array'
   * and call 'callback' when done
   *
   * @param {Array} array array
   * @param {Number} limit limit
   * @param {Function} iterator iterator
   * @param {Function} callback callbcak
   */

  async.eachLimit =
  async.parallel.eachLimit = function (array, limit, iterator, callback) {
    var completed = 0
      , started = 0
      , running = 0
      , len = array.length
      ;

    function complete (err) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      completed += 1;
      running -= 1;
      if (completed === len) {
        callback();
        return;
      }
    }

    while (running < limit && started < len) {
      iterator(array[started], complete);
      started += 1;
      running += 1;
    }
  };

  /**
   * Map 'iterator' to each item in 'array' in parallel
   * and call 'callback' when done
   *
   * @param {Array} array array
   * @param {Function} iterator iterator
   * @param {Function} callback callbcak
   */

  async.map =
  async.parallel.map = function (array, iterator, callback) {
    var results = []
      , completed = 0
      , len = array.length
      , i
      ;

    function complete (err, result) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      results[index] = result;
      completed += 1;
      if (completed === len) {
        callback(err, results);
        return;
      }
    }

    function iterate (index, value) {
      iterator(value, complete);
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterate(i, array[i]);
      }
    }
  };

  /**
   * Map 'iterator' to each item in 'array' in series
   * and call 'callback' when done
   *
   * @param {Array} array array
   * @param {Function} iterator iterator
   * @param {Function} callback callbcak
   */

  async.series.map = function (array, iterator, callback) {

    var results = []
      , completed = 0
      , len = array.length
      ;

    function iterate () {

      function complete (err, result) {
        if (err) {
          callback(err);
          callback = function () {};
          return;
        }
        results[completed] = result;
        completed += 1;
        if (completed === len) {
          callback(err, results);
          return;
        }
        iterate();
      }

      iterator(array[completed], complete);
    }

    iterate();
  };

}(this);