
/*!
 * async-util
 * JavaScript async utilities library
 *
 * Copyright (c) 2011 Enrico Marino
 * MIT license
 */

!function (exports) {

  exports.async = {};

  /**
   * Library version
   */
  async.version = 0.1.0;

  async.each_parallel = function (array, iterator, callback) {

    var completed = 0,
      len = array.length,
      i;

    function complete (err) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      completed += 1;
      if (completed === len) {
        callback();
        return;
      }
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterator(array[i], complete);
      }
    }
  }

  async.each_series = function (array, iterator, callback) {

    var completed = 0,
      len = array.length;

    function complete (err) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      completed += 1;
      if (completed === len) {
        callback();
        return;
      }
      iterate();
    }

    function iterate () {
      iterator(array[completed], complete);
    }

    iterate();
  }

  function each_limited (array, limit, iterator, callback) {

    var completed = 0,
      started = 0,
      running = 0,
      len = array.length;

    function complete (err) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      completed += 1;
      running -= 1;
      if (completed === len) {
        callback();
        return;
      }
    }

    while (running < limit && started < len) {
      iterator(array[started], complete);
      started += 1;
      running += 1;
    }
  }

  function map_parallel (array, iterator, callback) {

    var results = [],
      completed = 0,
      len = array.length,
      i;

    function iterate (index, value) {

      function complete (err, result) {
        if (err) {
          callback(err);
          callback = function () {};
          return;
        }
        results[index] = result;
        completed += 1;
        if (completed === len) {
          callback(err, results);
          return;
        }
      }

      iterator(value, complete);
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterate(i, array[i]);
      }
    }
  }

  function map_series (array, iterator, callback) {

    var results = [],
      completed = 0,
      len = array.length;

    function complete (err, result) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      results[completed] = result;
      completed += 1;
      if (completed === len) {
        callback(err, results);
        return;
      }
      iterate();
    }

    function iterate () {
      iterator(array[completed], complete);
    }

    iterate();
  }

  function reduce_series (array, memo, iterator, callback) {

    var completed = 0,
      len = array.length;

    function complete (err, result) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      memo = result;
      completed += 1;
      if (completed === len) {
        callback(err, result);
        return;
      }
      iterate();
    }

    function iterate () {
      iterator(memo, array[completed], complete);
    }

    iterate();
  }

  function filter_parallel (array, iterator, callback) {

    var concat = [].concat,
      results = [],
      completed = 0,
      len = array.length,
      i;

    function iterate (index, value) {

      function complete (err, result) {
        if (err) {
          callback(err);
          callback = function () {};
          return;
        }
        results[index] = result ? [value] : [];
        completed += 1;
        if (completed === len) {
          callback(concat.apply([], results));
          return;
        }
      }

      iterator(value, complete);
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterate(i, array[i]);
      }
    }
  }

  function filter_series (array, iterator, callback) {

    var results = [],
      completed = 0,
      len = array.length;

    function complete (err, result) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      if (result) {
        results.push(array[completed]);
      }
      completed += 1;
      if (completed === len) {
        callback(err, results);
        return;
      }
      iterate();
    }

    function iterate () {
      iterator(array[completed], complete);
    }

    iterate();
  }

  function reject_parallel (array, iterator, callback) {

    var concat = [].concat,
      results = [],
      completed = 0,
      len = array.length,
      i;

    function iterate (index, value) {

      function complete (err, result) {
        if (err) {
          callback(err);
          callback = function () {};
          return;
        }
        results[index] = result ? [] : [value];
        completed += 1;
        if (completed === len) {
          callback(concat.apply([], results));
          return;
        }
      }

      iterator(value, complete);
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterate(i, array[i]);
      }
    }
  }

  function reject_series (array, iterator, callback) {

    var results = [],
      completed = 0,
      len = array.length;

    function complete (err, result) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      if (!result) {
        results.push(array[completed]);
      }
      completed += 1;
      if (completed === len) {
        callback(err, results);
        return;
      }
      iterate();
    }

    function iterate () {
      iterator(array[completed], complete);
    }

    iterate();
  }

  function detect_parallel (array, iterator, callback) {

    var completed = 0,
      len = array.length,
      i;

    function complete (err, result) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      if (result) {
        callback(err, result);
        callback = function () {};
        return;
      }
      completed += 1;
      if (completed === len) {
        callback(err, result);
        return;
      }
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterator(array[i], complete);
      }
    }
  }

  function detect_series (array, iterator, callback) {

    var completed = 0,
      len = array.length;

    function complete (err, result) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      if (result) {
        callback(err, result);
        callback = function () {};
        return;
      }
      completed += 1;
      if (completed === len) {
        callback(err, result);
        return;
      }
      iterate();
    }

    function iterate () {
      iterator(array[completed], complete);
    }

    iterate();
  }

  function some (array, iterator, callback) {

    var completed = 0,
      len = array.length,
      i;

    function complete (err, result) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      if (result) {
        callback(err, true);
        callback = function () {};
        return;
      }
      completed += 1;
      if (completed === len) {
        callback(err, false);
        return;
      }
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterator(array[i], complete);
      }
    }
  }

  function every (array, iterator, callback) {

    var completed = 0,
      len = array.length,
      i;

    function complete (err, result) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      if (!result) {
        callback(err, false);
        callback = function () {};
        return;
      }
      completed += 1;
      if (completed === len) {
        callback(err, true);
        return;
      }
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterator(array[i], complete);
      }
    }
  }

  return {
    each_parallel: each_parallel,
    each_series: each_series,
    each_limited: each_limited,
    map_parallel: map_parallel,
    map_series: map_series,
    reduce_series: reduce_series,
    filter_parallel: filter_parallel,
    filter_series: filter_series,
    reject_parallel: reject_parallel,
    reject_series: reject_series,
    detect_parallel: detect_parallel,
    detect_series: detect_series,
    some: some,
    every: every
  };

}(this));